import argparse
import os
import json
from mail import generate_mail, save_mail, open_connection, close_connection, send_mail
from sampling import HamiltonSampler

SAMPLERS = {"hamilton": HamiltonSampler}
MAILDIR = "./mails"
os.makedirs(MAILDIR, exist_ok=True)
MAIL_TEMPLATE = "./src/assignment_mail.txt"

def prepare_mail_content(giver, presentee):
    """Fills the template mail text with the respective attributes, most
    importantly the address of the presentee.

    Args:
        giver: deserialized JSON representation of the person making the gift
        presentee: deserialized JSON representation of the receiver of the gift

    Returns:
        str: the completed mail body
    """
    with open(MAIL_TEMPLATE, "r") as f:
        text = f.read()
    text = text.replace("{giver_name}", giver["name"])
    for attribute in ["name", "fullname", "street", "city", "address_appendix"]:
        value = presentee[attribute] if attribute in presentee else ""
        text = text.replace(f"{{presentee_{attribute}}}", value)
    text = text.replace("{presentee_note}", f"Notiz der Person: {presentee['note']}" if "note" in presentee else "")
    return text


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("input", 
                        help="the input file containing the data of the participants")
    parser.add_argument("--sampling", 
                        choices=list(SAMPLERS.keys()),
                        default="hamilton",
                        help="The sampling strategy/algorithm for choosing pairs")
    args = parser.parse_args()


    # Load the people
    with open(args.input, "r") as f:
        people = json.load(f)
    print(f"Loaded {len(people)} participants")

    # Sample the pairs
    sampler = SAMPLERS[args.sampling](list(people.keys()))
    pairs = sampler.sample_pairs()
    with open("pairs.dump", "w") as f:
        json.dump(pairs, f, indent=4, ensure_ascii=False)

    # Generate mails
    mails = []
    for pair in pairs:
        giver, presentee = people[pair[0]], people[pair[1]]
        mail_content = prepare_mail_content(giver, presentee)
        mail = generate_mail(pair[0], mail_content)
        mails.append(mail)
        save_mail(f"{MAILDIR}/{pair[0]}.eml", mail)

    # Ask for confirmation
    print(f"Created {len(mails)} mails. Please check them in the folder {MAILDIR}.")
    command = input(f"Enter \"confirm\" to automatically send the mails. Please don't terminate the program, or the pairs will be lost.\n")

    if command != "confirm":
        print("Aborting...")
        exit()

    # Send the mails
    print("Sending mails...")
    open_connection()
    for msg in mails:
        send_mail(msg, auto_close_connection=False)
    close_connection()