"""This module provides various (currently only one) strategies to draw pairs
for the secret-Knecht-Robobert game. When implementing new samplers, don't
forget to add them to the SAMPLERS dict in the main file.
"""

import random

class Sampler():
    """Parent class for all samplers implemented.

    Each sampler should be initialized for a set of elements to draw from, and
    should implement a method to sample a full set of pairs.
    """
    def __init__(self, elements):
        self.elements = elements

    def sample_pairs(self):
        raise NotImplementedError()

class HamiltonSampler(Sampler):
    """Probably the easiest solution is to implement generating a Hamiltonian
    path, which visits every node of a graph exactly once. This implementation
    works by shuffling the list of elements, and then letting the first person
    make a gift for the second person, the second for the third, ..., and the
    last person for the first one.

    https://en.wikipedia.org/wiki/Hamiltonian_path
    """
    def sample_pairs(self):
        pairs = []
        e = self.elements[:]
        random.shuffle(e)
        for i in range(len(e)):
            pairs.append((e[i], e[i+1 if i+1 < len(e) else 0]))
        return pairs
        